@@ -31,6 +31,7 @@
 void runVCFTester(GraphCompareParameters& parameters);
 void runGraphDiff(GraphCompareParameters& parameters);
 void runDebug(GraphCompareParameters& parameters);
+void runInteractive(GraphCompareParameters& parameters);
 void preloadBloomFilter(const ReadTable* pReadTable, size_t k, BloomFilter* pBloomFilter);
 
 // Defines to clarify awful template function calls
@@ -76,8 +77,7 @@ static const char *GRAPH_DIFF_USAGE_MESSAGE =
 "Algorithm options:\n"
 "      -k, --kmer=K                     use K-mers to discover variants\n"
 "      -x, --min-discovery-count=T      require a variant k-mer to be seen at least T times\n"
-"          --debruijn                   use the de Bruijn graph assembly algorithm (default: string graph)\n"
-"          --paired-debruijn            use the de Bruijn graph assembly algorithm with paired-end constraints(default: string graph)\n"
+"      -a, --algorithm=STR              select the assembly algorithm to use from: debruijn, string\n"
 "      -m, --min-overlap=N              require at least N bp overlap when assembling using a string graph\n" 
 "          --min-dbg-count=T            only use k-mers seen T times when assembling using a de Bruijn graph\n"
 "\nReport bugs to " PACKAGE_BUGREPORT "\n\n";
@@ -99,11 +99,11 @@ namespace opt
     static int minDBGCount = 2;
 
     // Calling modes
-    static bool deBruijnMode = false;
-    static bool deBruijnPairedMode = false;
+    static GraphCompareAlgorithm algorithm = GCA_DEBRUIJN_GRAPH;
     static bool lowCoverage = false;
     static bool referenceMode = false;
     static bool useQualityScores = false;
+    static bool interactiveMode = false;
 
     // I/O
     static std::string outPrefix = "sgavariants";
@@ -115,7 +115,7 @@ namespace opt
     static std::string inputVCFFile;
 }
 
-static const char* shortopts = "b:r:o:k:d:t:x:y:p:m:v";
+static const char* shortopts = "b:r:o:k:d:t:x:y:p:m:a:v";
 
 enum { OPT_HELP = 1, 
        OPT_VERSION, 
@@ -124,12 +124,11 @@ enum { OPT_HELP = 1,
        OPT_DEBUG, 
        OPT_MIN_DBG_COUNT, 
        OPT_INDEX, 
-       OPT_DEBRUIJN, 
-       OPT_PAIRED_DEBRUIJN, 
        OPT_LOWCOVERAGE, 
        OPT_QUALSCORES,
        OPT_BLOOM_GENOME,
-       OPT_PRECACHE_REFERENCE };
+       OPT_PRECACHE_REFERENCE,
+       OPT_INTERACTIVE };
 
 static const struct option longopts[] = {
     { "verbose",              no_argument,       NULL, 'v' },
@@ -141,10 +140,10 @@ static const struct option longopts[] = {
     { "sample-rate",          required_argument, NULL, 'd' },
     { "prefix",               required_argument, NULL, 'p' },
     { "min-overlap",          required_argument, NULL, 'm' },
-    { "debruijn",             no_argument,       NULL, OPT_DEBRUIJN },
-    { "paired-debruijn",      no_argument,       NULL, OPT_PAIRED_DEBRUIJN },
+    { "algorithm",            required_argument, NULL, 'a' },
     { "low-coverage",         no_argument,       NULL, OPT_LOWCOVERAGE },
     { "use-quality-scores",   no_argument,       NULL, OPT_QUALSCORES},
+    { "interactive",          no_argument,       NULL, OPT_INTERACTIVE},
     { "index",                required_argument, NULL, OPT_INDEX },
     { "min-dbg-count",        required_argument, NULL, OPT_MIN_DBG_COUNT },
     { "debug",                required_argument, NULL, OPT_DEBUG },
@@ -248,14 +247,7 @@ int graphDiffMain(int argc, char** argv)
     sharedParameters.referenceIndex = referenceIndex;
     sharedParameters.pRefTable = &refTable;
     sharedParameters.bReferenceMode = opt::referenceMode;
-
-    if(opt::deBruijnPairedMode)
-        sharedParameters.algorithm = GCA_PAIRED_DEBRUIJN_GRAPH;
-    else if(opt::deBruijnMode)
-        sharedParameters.algorithm = GCA_DEBRUIJN_GRAPH;
-    else
-        sharedParameters.algorithm = GCA_STRING_GRAPH;
-
+    sharedParameters.algorithm = opt::algorithm;
     sharedParameters.kmer = opt::kmer;
     sharedParameters.minDiscoveryCount = opt::minDiscoveryCount;
     sharedParameters.minDBGCount = opt::minDBGCount;
@@ -278,6 +270,10 @@ int graphDiffMain(int argc, char** argv)
     {
         runDebug(sharedParameters);
     }
+    else if(opt::interactiveMode)
+    {
+        runInteractive(sharedParameters);
+    }
     else
     {
         // If a VCF file was provided, just test the variants in the file without any discovery
@@ -445,11 +441,54 @@ void runDebug(GraphCompareParameters& parameters)
     graphCompare.testKmersFromFile(opt::debugFile);
 }
 
+// Run interactively, parse a sequence from cin, process it then wait
+void runInteractive(GraphCompareParameters& parameters)
+{
+    // Initialize a bloom filter
+    size_t expected_bits = 0;
+    if(opt::bloomGenomeSize == -1)
+    {
+        // Calculate the expected bits using the number of bases in the reference
+        for(size_t i = 0; i < parameters.pRefTable->getCount(); ++i)
+            expected_bits += parameters.pRefTable->getReadLength(i);
+    }
+    else 
+    {
+        expected_bits = opt::bloomGenomeSize;
+    }
+
+    size_t occupancy_factor = 20;
+    size_t bloom_size = occupancy_factor * expected_bits;
+
+    BloomFilter* pBloomFilter = new BloomFilter(bloom_size, 5);
+    parameters.pBloomFilter = pBloomFilter;
+    preloadBloomFilter(parameters.pRefTable, parameters.kmer, pBloomFilter);
+    
+    GraphCompare graphCompare(parameters); 
+    while(1) 
+    {
+        std::string sequence;
+        std::cout << "Input a sequence: ";
+        std::cin >> sequence;
+
+        if(sequence == "done")
+            break;
+        SequenceWorkItem item;
+        item.read.seq = sequence;
+        item.read.id = "input";
+
+        std::cout << "processing: " << sequence << "\n";
+
+        graphCompare.process(item);
+    }
+}
+
 // 
 // Handle command line arguments
 //
 void parseGraphDiffOptions(int argc, char** argv)
 {
+    std::string algorithmString;
     bool die = false;
     for (char c; (c = getopt_long(argc, argv, shortopts, longopts, NULL)) != -1;) 
     {
@@ -464,11 +503,10 @@ void parseGraphDiffOptions(int argc, char** argv)
             case 'd': arg >> opt::sampleRate; break;
             case 'p': arg >> opt::outPrefix; break;
             case 'm': arg >> opt::minOverlap; break;
+            case 'a': arg >> algorithmString; break;
             case '?': die = true; break;
             case 'v': opt::verbose++; break;
             case OPT_REFERENCE: arg >> opt::referenceFile; break;
-            case OPT_DEBRUIJN: opt::deBruijnMode = true; break;
-            case OPT_PAIRED_DEBRUIJN: opt::deBruijnPairedMode = true; break;
             case OPT_LOWCOVERAGE: opt::lowCoverage = true; break;
             case OPT_MIN_DBG_COUNT: arg >> opt::minDBGCount; break;
             case OPT_BLOOM_GENOME: arg >> opt::bloomGenomeSize; break;
@@ -477,6 +515,7 @@ void parseGraphDiffOptions(int argc, char** argv)
             case OPT_TESTVCF: arg >> opt::inputVCFFile; break;
             case OPT_INDEX: arg >> opt::indexPrefix; break;
             case OPT_QUALSCORES:  opt::useQualityScores = true; break;
+            case OPT_INTERACTIVE:  opt::interactiveMode = true; break;
             case OPT_HELP:
                 std::cout << GRAPH_DIFF_USAGE_MESSAGE;
                 exit(EXIT_SUCCESS);
@@ -508,6 +547,17 @@ void parseGraphDiffOptions(int argc, char** argv)
     if(opt::baseFile.empty())
         opt::referenceMode = true;
 
+    if(algorithmString == "paired") {
+        opt::algorithm = GCA_PAIRED_DEBRUIJN_GRAPH;
+    } else if(algorithmString == "debruijn") {
+        opt::algorithm = GCA_DEBRUIJN_GRAPH;
+    } else if(algorithmString == "string") {
+        opt::algorithm = GCA_STRING_GRAPH;
+    } else {
+        std::cerr << "Error: unrecognized algorithm string: " << algorithmString << "\n";
+        die = true;
+    }
+
     if(opt::referenceFile.empty())
     {
         std::cerr << SUBPROGRAM ": error, a --reference file must be provided\n";
