@@ -36,6 +36,7 @@ static const char *PREPROCESS_USAGE_MESSAGE =
 "\n"
 "      --help                           display this help and exit\n"
 "      -v, --verbose                    display verbose output\n"
+"          --seed                       set random seed\n"
 "\nInput/Output options:\n"
 "      -o, --out=FILE                   write the reads to FILE (default: stdout)\n"
 "      -p, --pe-mode=INT                0 - do not treat reads as paired (default)\n"
@@ -45,6 +46,7 @@ static const char *PREPROCESS_USAGE_MESSAGE =
 "          --pe-orphans=FILE            if one half of a read pair fails filtering, write the passed half to FILE\n"
 "\nConversions/Filtering:\n"
 "          --phred64                    convert quality values from phred-64 to phred-33.\n"
+"          --discard-quality            do not output quality scores\n"
 "      -q, --quality-trim=INT           perform Heng Li's BWA quality trim algorithm. \n"
 "                                       Reads are trimmed according to the formula:\n"
 "                                       argmax_x{\\sum_{i=x+1}^l(INT-q_i)} if q_l<INT\n"
@@ -80,6 +82,7 @@ enum QualityScaling
 namespace opt
 {
     static unsigned int verbose;
+    static unsigned int seed = 0;
     static std::string outFile;
     static unsigned int qualityTrim = 0;
     static unsigned int hardClip = 0;
@@ -89,6 +92,7 @@ namespace opt
     static double sampleFreq = 1.0f;
 
     static bool bDiscardAmbiguous = true;
+    static bool bDiscardQuality = false;
     static QualityScaling qualityScale = QS_SANGER;
 
     static bool bFilterGC = false;
@@ -106,8 +110,11 @@ namespace opt
 
 static const char* shortopts = "o:q:m:h:p:r:c:s:f:vi";
 
-enum { OPT_HELP = 1, OPT_VERSION, OPT_PERMUTE, OPT_QSCALE, OPT_MINGC, OPT_MAXGC, 
-       OPT_DUST, OPT_DUST_THRESHOLD, OPT_SUFFIX, OPT_PHRED64, OPT_OUTPUTORPHANS, OPT_DISABLE_PRIMER };
+enum { OPT_HELP = 1, OPT_SEED, OPT_VERSION, OPT_PERMUTE,
+       OPT_QSCALE, OPT_MINGC, OPT_MAXGC,
+       OPT_DUST, OPT_DUST_THRESHOLD, OPT_SUFFIX,
+       OPT_PHRED64, OPT_OUTPUTORPHANS, OPT_DISABLE_PRIMER,
+       OPT_DISCARD_QUALITY };
 
 static const struct option longopts[] = {
     { "verbose",                no_argument,       NULL, 'v' },
@@ -130,7 +137,9 @@ static const struct option longopts[] = {
     { "help",                   no_argument,       NULL, OPT_HELP },
     { "version",                no_argument,       NULL, OPT_VERSION },
     { "permute-ambiguous",      no_argument,       NULL, OPT_PERMUTE },
+    { "discard-quality",        no_argument,       NULL, OPT_DISCARD_QUALITY },
     { "no-primer-check",        no_argument,       NULL, OPT_DISABLE_PRIMER },
+    { "seed",                   required_argument, NULL, OPT_SEED },
     { NULL, 0, NULL, 0 }
 };
 
@@ -150,6 +159,12 @@ int preprocessMain(int argc, char** argv)
     Timer* pTimer = new Timer("sga preprocess");
     parsePreprocessOptions(argc, argv);
 
+    // set random seed
+    if (opt::seed == 0)
+    {
+        opt::seed = time(NULL);
+    }
+
     std::cerr << "Parameters:\n";
     std::cerr << "QualTrim: " << opt::qualityTrim << "\n";
 
@@ -169,6 +184,8 @@ int preprocessMain(int argc, char** argv)
     std::cerr << "Orphan file: " << (opt::orphanFile.empty() ? "none" : opt::orphanFile) << "\n";
     if(opt::bDiscardAmbiguous)
         std::cerr << "Discarding sequences with ambiguous bases\n";
+    if(opt::bDiscardQuality)
+        std::cerr << "Discarding quality scores\n";
     if(opt::bDustFilter)
         std::cerr << "Dust threshold: " << opt::dustThreshold << "\n";
     if(!opt::suffix.empty())
@@ -179,9 +196,10 @@ int preprocessMain(int argc, char** argv)
         std::cerr << "Adapter sequence fwd: " << opt::adapterF << "\n";
         std::cerr << "Adapter sequence rev: " << opt::adapterR << "\n";
     }
+    std::cerr << "Seed: " << opt::seed << "\n";
 
     // Seed the RNG
-    srand(time(NULL));
+    srand(opt::seed);
 
     std::ostream* pWriter;
     if(opt::outFile.empty())
@@ -244,6 +262,13 @@ int preprocessMain(int argc, char** argv)
                 // Read from separate files
                 std::string filename1 = argv[optind++];
                 std::string filename2 = argv[optind++];
+                
+                if(filename1 == "-" || filename2 == "-")
+                {
+                    std::cerr << "Reading from stdin is not supported in --pe-mode 1\n";
+                    std::cerr << "Maybe you meant --pe-mode 2 (interleaved pairs?)\n";
+                    exit(EXIT_FAILURE);
+                }
 
                 pReader1 = new SeqReader(filename1, SRF_NO_VALIDATION);
                 pReader2 = new SeqReader(filename2, SRF_NO_VALIDATION);
@@ -321,7 +346,6 @@ int preprocessMain(int argc, char** argv)
             }
             delete pReader1;
             pReader1 = NULL;
-
         }
 
     }
@@ -500,7 +524,11 @@ bool processRead(SeqRecord& record)
     }
 
     record.seq = seqStr;
-    record.qual = qualStr;
+
+    if(opt::bDiscardQuality)
+        record.qual.clear();
+    else
+        record.qual = qualStr;
 
     if(record.seq.length() == 0 || record.seq.length() < opt::minLength)
         return false;
@@ -611,6 +639,8 @@ void parsePreprocessOptions(int argc, char** argv)
             case OPT_PERMUTE: opt::bDiscardAmbiguous = false; break;
             case OPT_DUST: opt::bDustFilter = true; break;
             case OPT_DISABLE_PRIMER: opt::bDisablePrimerCheck = true; break;
+            case OPT_DISCARD_QUALITY: opt::bDiscardQuality = true; break;
+            case OPT_SEED: arg >> opt::seed; break;
             case OPT_HELP:
                 std::cout << PREPROCESS_USAGE_MESSAGE;
                 exit(EXIT_SUCCESS);
